@@ -15,7 +15,7 @@ use Lemonldap::NG::Common::Apache::Session::Serialize::JSON;
 use Lemonldap::NG::Common::Apache::Session::Store;
 use Lemonldap::NG::Common::Apache::Session::Lock;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.19.0';
 
 sub _load {
     my ( $backend, $func ) = @_;
@@ -61,7 +61,9 @@ sub populate {
     }
 
     # If cache is configured, use our specific object store module
-    if ( $> and $self->{args}->{localStorage} ) {
+    if ( ( $> or $self->{args}->{allow_cache_for_root} )
+        and $self->{args}->{localStorage} )
+    {
         $self->{args}->{object_store} = $self->{object_store};
         $self->{object_store} =
           Lemonldap::NG::Common::Apache::Session::Store->new($self);
@@ -135,6 +137,29 @@ sub searchOnExpr {
     return \%res;
 }
 
+sub searchGt {
+    my ( $class, $args, $selectField, $value, @fields ) = splice @_;
+    return $args->{backend}->searchGt( $args, $selectField, $value, @fields )
+      if ( _load( $args->{backend}, 'searchGt' ) );
+    my %res = ();
+    $class->get_key_from_all_sessions(
+        $args,
+        sub {
+            my $entry = shift;
+            my $id    = shift;
+            return undef unless ( $entry->{$selectField} > $value );
+            if (@fields) {
+                $res{$id}->{$_} = $entry->{$_} foreach (@fields);
+            }
+            else {
+                $res{$id} = $entry;
+            }
+            undef;
+        }
+    );
+    return \%res;
+}
+
 sub searchLt {
     my ( $class, $args, $selectField, $value, @fields ) = splice @_;
     return $args->{backend}->searchLt( $args, $selectField, $value, @fields )
@@ -183,9 +208,6 @@ sub get_key_from_all_sessions {
     {
         return $class->_dbiGKFAS( $1, @_ );
     }
-    elsif ( $backend =~ /^Apache::Session::(?:NoSQL|Redis|Cassandra)$/ ) {
-        return $class->_NoSQLGKFAS(@_);
-    }
     elsif ( $backend =~ /^Apache::Session::(File|PHP|DBFile|LDAP)$/ ) {
         no strict 'refs';
         my $tmp = "_${1}GKFAS";
@@ -394,30 +416,4 @@ sub _LDAPGKFAS {
     return \%res;
 }
 
-sub _NoSQLGKFAS {
-    my ( $class, $args, $data ) = @_;
-    require Redis;
-    die "Only Redis is supported" unless ( $args->{Driver} eq 'Redis' );
-    my $redis = Redis->new(%$args);
-    my @keys  = $redis->keys('*');
-    my %res;
-
-    foreach my $k (@keys) {
-        my $v =
-          eval { $args->{unserialize}->( $redis->get($k), \&decodeThaw64 ); };
-        next if ($@);
-        if ( ref($data) eq 'CODE' ) {
-            $res{$k} = &$data( $v, $k );
-        }
-        elsif ($data) {
-            $data = [$data] unless ( ref($data) );
-            $res{$k}->{$_} = $v->{$_} foreach (@$data);
-        }
-        else {
-            $res{$k} = $v;
-        }
-    }
-    return \%res;
-}
-
 1;
